/**************************************************************************************
Author:         Dave Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Date:           11/19/2009
Program:        materials_arch_design
Description:    This script allows the user to affect muliple arch design materials
                simultaneously
***************************************************************************************/

try (destroyDialog arch_material_rollout) catch()


struct ArchDesignMaterials
(
    arch_materials = #(),
    arch_material_names = #(),
    arch_numeric_properties = #(),
    arch_boolean_properties = #(),
    arch_color_properties = #(),
    arch_map_properties = #(),
    arch_map_multiplier_properties = #(),
    arch_map_boolean_properties = #(),
	
	/**************************************************************************
    Sorts items by name
    **************************************************************************/
    fn sort_by_name item_one item_two =
    (
        case of
        (
            (item_one.name < item_two.name): -1
            (item_one.name > item_two.name): 1
            default: 0
        ) --end case
    ), --end fn
    
    /**************************************************************************
    Collects a list of all arch design materials and their names in the
    current scene
    **************************************************************************/
    fn collect_arch_materials =
    (
        this.arch_materials = getClassInstances Arch___Design__mi
        
        qsort this.arch_materials this.sort_by_name
        
        this.arch_material_names = for current in this.arch_materials collect current.name
    ), --end fn
    
    /**************************************************************************
    Collects a list of all vray material properties and sorts them
    **************************************************************************/
    fn collect_arch_properties =
    (
        local temp_material = Arch___Design__mi()
        
        --CRITICAL! Delete previous data!
        this.arch_numeric_properties = #()
        this.arch_boolean_properties = #()
        this.arch_color_properties = #()
        this.arch_map_properties = #()
        this.arch_map_multiplier_properties = #()
        this.arch_map_boolean_properties = #()
        
        for current_property in (getPropNames temp_material) do
        (
            if (matchPattern current_property pattern:"*_map*" ignorecase:true) then
            (
                case (classOf (getProperty temp_material current_property)) of
                (
                    (Integer): append arch_map_multiplier_properties (current_property as string)
                    (Float): append arch_map_multiplier_properties (current_property as string)
                    (BooleanClass): append arch_map_boolean_properties (current_property as string)
                    (UndefinedClass): append arch_map_properties (current_property as string)
                    default: print("WARNING: Unknown property found: " + (current_property as string))
                ) --end case
            ) --end if
            else
            (
                case (classOf (getProperty temp_material current_property)) of
                (
                    (Integer): append arch_numeric_properties (current_property as string)
                    (Float): append arch_numeric_properties (current_property as string)
                    (BooleanClass): append arch_boolean_properties (current_property as string)
                    (Color): append arch_color_properties (current_property as string)
                    default: if not (matchPattern current_property pattern:"mapM*" ignorecase:true) do
                                print("WARNING: Unknown property found: " + (current_property as string))
                ) --end case
            ) --end else
        ) --end for
        
        sort this.arch_numeric_properties
        sort this.arch_boolean_properties
        sort this.arch_color_properties
        sort this.arch_map_properties
        sort this.arch_map_multiplier_properties
        sort this.arch_map_boolean_properties
    ), --end fn
    
    /**************************************************************************
    Sets the material property 'property_name' to the value of 'property_value'
    for the materials defined by 'mode'
    **************************************************************************/
    fn set_material_property mode user_selection property_name property_value =
    (
        case mode of
        (
            (1): for index in user_selection do
                    setProperty this.arch_materials[index] property_name property_value
            (2): for current in selection do
                    if current.material != undefined do
                        if (classOf current.material) == Arch___Design__mi do
                            setProperty current.material property_name property_value
            (3): for current in this.arch_materials do
                    setProperty current property_name property_value
        ) --end case
    ) --end fn
) --end struct


( --local scope
    global arch_material_rollout
    local arch_material_ini = getDir #plugcfg + "\\ArchDesignMaterials.ini"
    local arch_material_position
    
    rollout arch_material_rollout "Arch n Design Materials"
    (
        local arch_mats = ArchDesignMaterials()
        
        /***********************************************************************
        Enables/Disables the UI controls based on argument 'state'
        ***********************************************************************/
        fn enable_controls state =
        (
            arch_material_rollout.numeric_properties_dropdown.enabled = state
            arch_material_rollout.numeric_properties_spinner.enabled = state
            arch_material_rollout.boolean_properties_dropdown.enabled = state
            arch_material_rollout.boolean_true_button.enabled = state
            arch_material_rollout.boolean_false_button.enabled = state
            arch_material_rollout.color_properties_dropdown.enabled = state
            arch_material_rollout.color_properties_picker.enabled = state
            arch_material_rollout.map_properties_dropdown.enabled = state
            arch_material_rollout.map_properties_button.enabled = state
            arch_material_rollout.map_multiplier_dropdown.enabled = state
            arch_material_rollout.map_multiplier_spinner.enabled = state
            arch_material_rollout.map_boolean_dropdown.enabled = state
            arch_material_rollout.map_boolean_true_button.enabled = state
            arch_material_rollout.map_boolean_false_button.enabled = state
        ) --end fn
        
        /***********************************************************************
        Refreshes the material names in the list box
        ***********************************************************************/
        fn refresh_arch_material_list =
        (
            arch_mats.collect_arch_materials()
            
            message = "Total: " + (arch_mats.arch_materials.count as string)
            arch_material_rollout.total_label.text = message
            
            arch_material_rollout.material_listbox.items = arch_mats.arch_material_names
        ) --end fn
        
        /***********************************************************************
        Refreshes the property names in the property dropdowns
        ***********************************************************************/
        fn refresh_arch_property_dropdowns =
        (
            arch_mats.collect_arch_properties()
            
            arch_material_rollout.numeric_properties_dropdown.items = arch_mats.arch_numeric_properties
            arch_material_rollout.boolean_properties_dropdown.items = arch_mats.arch_boolean_properties
            arch_material_rollout.color_properties_dropdown.items = arch_mats.arch_color_properties
            arch_material_rollout.map_properties_dropdown.items = arch_mats.arch_map_properties
            arch_material_rollout.map_multiplier_dropdown.items = arch_mats.arch_map_multiplier_properties
            arch_material_rollout.map_boolean_dropdown.items = arch_mats.arch_map_boolean_properties
        ) --end fn
        
        /***********************************************************************
        Returns 'OK' if the user selection and mode are valid. Error message
        otherwise
        ***********************************************************************/
        fn get_validation =
        (
            local user_selection = arch_material_rollout.material_listbox.selection as array
            local validation = "OK"
            
            case arch_material_rollout.mode_radio.state of
            (
                (1): if user_selection.count == 0 do validation = "ERROR: No materials selected"
                (2): if selection.count == 0 do validation = "ERROR: No objects selected"
                (3): if arch_mats.arch_materials.count == 0 do validation = "ERROR: No arch design materials found"
            ) --end case
            
            validation --return value
        ) --end fn

        /***********************************************************************
        UI creation
        ***********************************************************************/
        label total_label "Total: 0" pos:[5,5] width:100 height:15
        button update_button "Update" pos:[245,3] width:50 height:15
        multiListBox material_listbox "" pos:[5,20] width:290 height:17
        
        groupBox mode_group "Mode" pos:[5,250] width:290 height:55
            radioButtons mode_radio "" pos:[15,270] width:174 height:32 labels:#("List Select", "Scene Select", "All Materials") columns:2 default:1
            
        groupBox property_group "Properties" pos:[5,310] width:290 height:200
            dropDownList numeric_properties_dropdown "" pos:[15,330] width:200 height:21
            spinner numeric_properties_spinner "" pos:[225,332] width:60 height:15 range:[0,99999,1] type:#float
            dropDownList boolean_properties_dropdown "" pos:[15,360] width:200 height:21
            button boolean_true_button "ON" pos:[225,360] width:30 height:20
            button boolean_false_button "OFF" pos:[255,360] width:30 height:20
            dropDownList color_properties_dropdown "" pos:[15,390] width:200 height:21
            colorPicker color_properties_picker "" pos:[220,390] width:65 height:20
            dropDownList map_properties_dropdown "" pos:[15,420] width:200 height:21
            mapButton map_properties_button "M" pos:[225,420] width:60 height:20
            dropDownList map_multiplier_dropdown "" pos:[15,450] width:200 height:21
            spinner map_multiplier_spinner "" pos:[225,450] width:60 height:15 range:[0,99999,1] type:#float
            dropDownList map_boolean_dropdown "" pos:[15,480] width:200 height:21
            button map_boolean_true_button "ON" pos:[225,480] width:30 height:20
            button map_boolean_false_button "OFF" pos:[255,480] width:30 height:20
        
        label status_label "INFO: Standing by..." pos:[5,520] width:290 height:15
        
        /***********************************************************************
        Rollout open event handler. Prepares the rollout for first time open
        ***********************************************************************/
        on arch_material_rollout open do
        (
            if (classOf Arch___Design__mi) != material do
            (
                enable_controls false
                arch_material_rollout.status_label.text = "ERROR: Arch Design material not loaded. Check plugin status"
                return false
            ) --end if
            
            refresh_arch_material_list()
            refresh_arch_property_dropdowns()
            enable_controls true
        ) --end on

        /***********************************************************************
        Rollout moved event handler. Stores the current position of the rollout
        ***********************************************************************/
        on arch_material_rollout moved pos do
            setIniSetting arch_material_ini "Dialog" "Position" (pos as string)
        
        /***********************************************************************
        Update button clicked event handler. Updates the UI
        ***********************************************************************/
        on update_button pressed do
        (
            if (classOf Arch___Design__mi) != material do
            (
                enable_controls false
                arch_material_rollout.status_label.text = "ERROR: Arch design material not loaded. Check plugin status"
                return false
            ) --end if
            
            refresh_arch_material_list()
            refresh_arch_property_dropdowns()
            enable_controls true
            
            status_label.text = "INFO: Arch design material list updated"
        ) --end on

        /***********************************************************************
        Numeric spinner changed event handler. Applies the new numeric value to
        the selected property of the arch materials
        ***********************************************************************/
        on numeric_properties_spinner changed the_value do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                numeric_properties_dropdown.selected \
                                                the_value
                message = ("INFO: " + numeric_properties_dropdown.selected)
                message += (" set to " + (the_value as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Boolean ON clicked event handler. Turns on the selected property of the
        arch materials
        ***********************************************************************/
        on boolean_true_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                boolean_properties_dropdown.selected \
                                                true
                message = ("INFO: " + boolean_properties_dropdown.selected)
                message += " set to ON"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Boolean OFF clicked event handler. Turns off the selected property of the
        arch materials
        ***********************************************************************/
        on boolean_false_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                boolean_properties_dropdown.selected \
                                                false
                message = ("INFO: " + boolean_properties_dropdown.selected)
                message += " set to OFF"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Color picker clicked event handler. Changes the color of the selected
        property of the arch materials
        ***********************************************************************/
        on color_properties_picker changed the_color do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                color_properties_dropdown.selected \
                                                the_color
                message = ("INFO: " + color_properties_dropdown.selected)
                message += (" set to " + (the_color as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map picker clicked event handler. Assigns the map to the selected
        property of the arch materials
        ***********************************************************************/
        on map_properties_button picked the_map do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_properties_dropdown.selected \
                                                the_map
                message = ("INFO: " + map_properties_dropdown.selected)
                message += (" set to " + ((classOf the_map) as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map spinner changed event handler. Applies the new numeric value to
        the selected property of the arch materials
        ***********************************************************************/
        on map_multiplier_spinner changed the_value do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_multiplier_dropdown.selected \
                                                the_value
                message = ("INFO: " + map_multiplier_dropdown.selected)
                message += (" set to " + (the_value as string))
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map boolean ON clicked event handler. Turns on the selected property of
        the arch materials
        ***********************************************************************/
        on map_boolean_true_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_boolean_dropdown.selected \
                                                true
                message = ("INFO: " + map_boolean_dropdown.selected)
                message += " set to ON"
            ) --end if
            
            status_label.text = message
        ) --end on
        
        /***********************************************************************
        Map boolean OFF clicked event handler. Turns off the selected property
        of the arch materials
        ***********************************************************************/
        on map_boolean_false_button pressed do
        (
            local message = get_validation()
            
            if message == "OK" do
            (
                arch_mats.set_material_property mode_radio.state \
                                                (material_listbox.selection as array) \
                                                map_boolean_dropdown.selected \
                                                false
                message = ("INFO: " + map_boolean_dropdown.selected)
                message += " set to OFF"
            ) --end if
            
            status_label.text = message
        ) --end on
    ) --end rollout

    arch_material_position = execute (getIniSetting arch_material_ini "Dialog" "Position")
    if arch_material_position == OK do arch_material_position = [100,100]
    if arch_material_position.x > sysinfo.desktopsize.x - 300 do
        arch_material_position.x = sysinfo.desktopsize.x - 305
    if arch_material_position.y > sysinfo.desktopsize.y - 595 do
        arch_material_position.y = sysinfo.desktopsize.y - 595
    if arch_material_position.x < 0 do arch_material_position.x = 0
    if arch_material_position.y < 0 do arch_material_position.y = 0
    createDialog arch_material_rollout 300 535 arch_material_position.x arch_material_position.y style:#(#style_titlebar, #style_minimizebox, #style_border, #style_sysmenu)
) --end scope